/***************************************************************************************
Titel:		ATtiny TPS
Author:		Christian Hottum
File:		ATtiny461A_TPS.c
Version:	0.1
Datum:		09.01.2015
Software:	AVR-GCC 4.3.3
Target:		ATtiny461A

DESCRIPTION
	Firmware fr einen Tasten Programierbare Steuerung auf Basis einen ATtiny461A
	mit 3 Tasten (S1(PB0), S2(PB2) und reset(PB7)), 4 digitale Eingngen (PA0 - PA3), 
	2 ADC Eingnge (PB4 - PB5), 4 digitale Ausgnge (PA4 - PA7), 
	2 PWM Ausgnge (PB1, PB3), Freie Pins (PB6), 8 MHZ System Takt

***************************************************************************************/

// CPU Frequenz

#define F_CPU 8000000UL     /* Quarz mit 8 Mhz  */


// Include Dateien

#include <math.h>
#include <avr/io.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <util/delay.h>
#include "TPS_UP.h"
#include <avr/eeprom.h>

// Variablen

// I/O vorbereiten

int main(void)
{
	_delay_ms(200);


	// Ausgnge
	PORTA &= ~(1<<A1) | (1<<A2) | (1<<A3) | (1<<A4);	// alle Ausgnge aus
	DDRA |= (1<<A1) | (1<<A2) | (1<<A3) | (1<<A4); 		// als Ausgnge setzen


	// Tasten
	PORTB |= (1<<S1) | (1<<S2) | (1<<PINB6);			// Pull up Widerstnde an
	DDRB &= ~(1<<S1) | (1<<S2) | (1<<PINB6);			// als Eingnge setzen

	
	// Eingnge
	PORTA |= (1<<E1) | (1<<E2) | (1<<E3) | (1<<E4);		// Pull up Widerstnde an
	DDRA &= ~(1<<E1) | (1<<E2) | (1<<E3) | (1<<E4);		// als Eingnge setzen  


	// ADC's
	ADCSRA |= (1<<ADEN);								// ADC aktivieren
	ADMUX |= (1<<ADLAR);								// ADC Ausgabe left adjusted
	ADCSRA |= (1<<ADPS2) | (1<<ADPS0);					// ADC-Frequenz Clock/32 (verkrzt die Abfrage)


	// PWM's
	PORTB &= ~(1<<PB1) | (1<<PB3);						// alle Ausgnge aus
	DDRB |= (1<<PB1) | (1<<PB3);				 		// als Ausgnge setzen
	TCCR1B |= (1<<CS10);								// PWM-Mode setzen
	TCCR1D |= (1<<WGM10);								
	TCCR1D &= ~(1<<WGM11);
	TCCR1A |= (1<<PWM1A) | (1<<PWM1B) | (1<<COM1A1) | (1<<COM1B1);	
	

	// Hauptprogrammschleife

	while(1)
	{

		if (!(PINB & (1<<S2)))		// wenn S2 beim einschalten gedrckt ist
		{							// gehe in den Programmier-Modus
			PROG();
		}
		if (!(PINB & (1<<S1)))		// wenn S1 beim einschalten gedrckt ist
		{							// gehe in den Test-Modus
			TEST();
		}

		AUSF();						// ansonsten gehe in den Ausfhren-Modus

	}

	return 0;

}








