/***************************************************************************************
Titel:		ATtiny TPS Unterprogramme
Author:		Christian Hottum
File:		TPS_UP.h
Version:	0.1
Datum:		09.01.2015
Software:	AVR-GCC 4.3.3
Target:		ATtiny461A

DESCRIPTION
	Variablen und Unterprogramme der TPS 

***************************************************************************************/

// CPU Frequenz

#define F_CPU 8000000UL     /* Quarz mit 8 Mhz  */

// Include Dateien

#include <math.h>
#include <avr/io.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <util/delay.h>
#include <avr/eeprom.h>

// Tasten
#define S1			PINB0
#define S2			PINB2

// Eingnge
#define E1			PINA0
#define E2			PINA1
#define E3			PINA2
#define E4			PINA3

// Ausgnge
#define A1			PINA4
#define A2			PINA5
#define A3			PINA6
#define A4			PINA7

//PWM's
#define PWM1		OCR1A
#define PWM2		OCR1B


//Variablen
uint8_t		eebyte[256] EEMEM;		//Programmspeicher
uint8_t		byte = 0;				//Zwischenspeicher Programmbytes
uint8_t		addr = 0;				//Addresse Programmspeicher
uint8_t		dat = 0;				//Daten Programmspeicher
uint8_t		kom = 0;				//Befehl Programmspeicher
uint8_t		adrlo = 0;				//Adresse Low - unteres Nibble
uint8_t		adrhi = 0;				//Adresse High - oberes Nibble
uint8_t		adrret = 0;				//Rcksprungadresse
uint8_t		prog = 0;				//Programmierschritt
uint8_t 	puffer = 0;				//ADC Puffer fr ersten Wert
uint16_t	ff = 0;					//Rcksetzt Zhler
uint16_t	ms = 0;					//Zhlvariable langes Delay

uint8_t		A = 0;					//Speicher A
uint8_t		B = 0;					//Speicher B
uint8_t		C = 0;					//Speicher C
uint8_t		D = 0;					//Speicher D


// Unterprogramme

// ADC 1 auslesen
uint8_t ADC1(void)
{
	loop_until_bit_is_clear (ADCSRA, ADSC);		// warte bis das Bit ADSC gelscht wird
	ADMUX = 39;									// setze den Mux auf ADC 7
	ADCSRA |= (1<<ADSC);						// starte die Wandlung
	loop_until_bit_is_clear (ADCSRA, ADSC);		// warte bis das Bit ADSC gelscht wird
	return ADCH;								// gib den zweiten ADC-Wert zurck
}

// ADC 2 auslesen	
uint8_t ADC2(void)
{
	loop_until_bit_is_clear (ADCSRA, ADSC);		// warte bis das Bit ADSC gelscht wird
	ADMUX = 40;									// setze den Mux auf ADC 8
	ADCSRA |= (1<<ADSC);						// starte die Wandlung
	loop_until_bit_is_clear (ADCSRA, ADSC);		// warte bis das Bit ADSC gelscht wird		
	return ADCH;								// gib den zweiten ADC-Wert zurck
}

// Grere Delays
void long_delay(uint16_t ms)
{
    for(; ms>0; ms--) _delay_ms(1);
}


// Programm ausfhren
void AUSF()
{
	addr = 0;
	do
	{
		byte = eeprom_read_byte (&eebyte [addr]);	// lese das eebyte-Array an der stelle addr aus
		dat = byte & 15;						// schreibe das unter Nibble in dat und
		kom = byte / 16;						// das obere Nibble in kom
		addr = addr + 1;						// addiere 1 zu addr

		if (kom == 1)							// 1: Direkte Portausgabe
		{
			PORTA = dat * 16 & 0xF0;
		}

		if (kom == 2)							// 2: Wartezeit
		{
			if (dat == 0) _delay_ms(1);
			if (dat == 1) _delay_ms(2);
			if (dat == 2) _delay_ms(5);
			if (dat == 3) _delay_ms(10);
			if (dat == 4) _delay_ms(20);
			if (dat == 5) _delay_ms(50);
			if (dat == 6) _delay_ms(100);
			if (dat == 7) _delay_ms(200);			
			if (dat == 8) _delay_ms(500);
			if (dat == 9) _delay_ms(1000);
			if (dat == 10) _delay_ms(2000);
			if (dat == 11) _delay_ms(5000);
			if (dat == 12) long_delay(10000);
			if (dat == 13) long_delay(20000);
			if (dat == 14) long_delay(30000);
			if (dat == 15) long_delay(60000);
		}
		
		if (kom == 3)							// 3: Sprung - relativ
		{
			addr = addr - 1;
			addr = addr - dat;
		}
		
		if (kom == 4)							// 4: A = ...
		{
			A = dat;
		}
		
		if (kom == 5)							// 5: ... = A
		{
			if (dat == 1) B = A;				// Variablen
			if (dat == 2) C = A;
			if (dat == 3) D = A;
			if (dat == 4) PORTA = A * 16 & 0xF0;	// Port
			if (dat == 5)						// Portbits
			{
				if (A == 0) PORTA &= ~(1<<A1);
				else PORTA |= (1<<A1);
			}
			if (dat == 6)			
			{
				if (A == 0) PORTA &= ~(1<<A2);
				else PORTA |= (1<<A2);
			}
			if (dat == 7)
			{
				if (A == 0) PORTA &= ~(1<<A3);
				else PORTA |= (1<<A3);
			}			
			if (dat == 8)
			{
				if (A == 0) PORTA &= ~(1<<A4);
				else PORTA |= (1<<A4);
			}
			if (dat == 9) PWM1 = A * 16;		// PWM
			if (dat == 10) PWM2 = A * 16;
		}
		
		if (kom == 6)							// 6: A = ...
		{
			if (dat == 1) A = B;				// Variablen
			if (dat == 2) A = C;
			if (dat == 3) A = D;
			if (dat == 4) A = PINA & 15;		// Port
			if (dat == 5)						// Portbits
			{
				if (!(PINA & (1<<E1))) A = 0;
				else A = 1;
			}
			if (dat == 6)
			{
				if (!(PINA & (1<<E2))) A = 0;
				else A = 1;
			}
			if (dat == 7)
			{
				if (!(PINA & (1<<E3))) A = 0;
				else A = 1;
			}
			if (dat == 8)
			{
				if (!(PINA & (1<<E4))) A = 0;
				else A = 1;
			}
			if (dat == 9) A = ADC1() / 16;		// ADC
			if (dat == 10) A = ADC2() / 16;
		}

		if (kom == 7)							// 7: A = ...
		{
			if (dat == 1) A = A + 1;
			if (dat == 2) A = A - 1;
			if (dat == 3) A = A + B;
			if (dat == 4) A = A - B;
			if (dat == 5) A = A * B;
			if (dat == 6) A = A / B;
			if (dat == 7) A = A & B;			
			if (dat == 8) A = A | B;
			if (dat == 9) A = A ^ B;
			if (dat == 10) A = ~A;
		}
		
		if (kom == 8) adrhi = dat;				// 8: Oberes Nibble der Adresse
		
		if (kom == 9)							// 9: Spring absolut 0...255
		{
			addr = adrhi * 16;
			addr = addr + dat;
		}
		
		if (kom == 10)							// 10: C * Spring absolut 0...255
		{
			if (C > 0)
			{
				C = C - 1;
				C = C & 15;
				addr = adrhi * 16;
				addr = addr + dat;
			}
		}
		
		if (kom == 11)							// 11: D * Spring absolut 0...255
		{
			if (D > 0)
			{
				D = D - 1;
				D = D & 15;
				addr = adrhi * 16;
				addr = addr + dat;
			}
		}		
		
		if (kom == 12)							// 12: Skip if ...
		{
			if (dat == 1)
			{
				if (A > B) addr = addr + 1;
			}
			if (dat == 2)
			{
				if (A < B) addr = addr + 1;
			}			
			if (dat == 3)
			{
				if (A == B) addr = addr + 1;
			}
			if (dat == 4)
			{
				if (PINA & (1<<E1)) addr = addr + 1;
			}
			if (dat == 5)
			{
				if (PINA & (1<<E2)) addr = addr + 1;
			}
			if (dat == 6)
			{
				if (PINA & (1<<E3)) addr = addr + 1;
			}
			if (dat == 7)
			{
				if (PINA & (1<<E4)) addr = addr + 1;
			}
			if (dat == 8)
			{
				if (!(PINA & (1<<E1))) addr = addr + 1;
			}
			if (dat == 9)
			{
				if (!(PINA & (1<<E2))) addr = addr + 1;
			}
			if (dat == 10)
			{
				if (!(PINA & (1<<E3))) addr = addr + 1;
			}
			if (dat == 11)
			{
				if (!(PINA & (1<<E4))) addr = addr + 1;
			}
			if (dat == 12)
			{
				if (!(PINB & (1<<S1))) addr = addr + 1;
			}
			if (dat == 13)
			{
				if (!(PINB & (1<<S2))) addr = addr + 1;
			}
			if (dat == 14)
			{
				if (PINB & (1<<S1)) addr = addr + 1;
			}
			if (dat == 15)
			{
				if (PINB & (1<<S2)) addr = addr + 1;
			}
		}
		
		if (kom == 13)							// 13: Call Unterprogramm absolut 0...255
		{
			adrret = addr;
			addr = adrhi * 16;
			addr = addr + dat;
		}
		
		if (kom == 14)							// 14: Return
		{
			addr = adrret;
		}
		
		if (kom == 15 )							// 15: EEProm neu schreiben
		{
			if ((dat == 15) && (addr < 3))
			{
				ff++;
				if (ff == 2)
				{
					eeprom_write_byte (&eebyte [0], 0x64);
					eeprom_write_byte (&eebyte [1], 0x51);
					eeprom_write_byte (&eebyte [2], 0x4E);
					eeprom_write_byte (&eebyte [3], 0x80);
					eeprom_write_byte (&eebyte [4], 0xC3);
					eeprom_write_byte (&eebyte [5], 0x98);
					eeprom_write_byte (&eebyte [6], 0x82);
					eeprom_write_byte (&eebyte [7], 0x95);
					eeprom_write_byte (&eebyte [8], 0x4D);
					eeprom_write_byte (&eebyte [9], 0x80);
					eeprom_write_byte (&eebyte [10], 0xC3);
					eeprom_write_byte (&eebyte [11], 0x9E);
					eeprom_write_byte (&eebyte [12], 0x82);
					eeprom_write_byte (&eebyte [13], 0x9A);
					eeprom_write_byte (&eebyte [14], 0x4B);
					eeprom_write_byte (&eebyte [15], 0x81);
					eeprom_write_byte (&eebyte [16], 0xC3);
					eeprom_write_byte (&eebyte [17], 0x94);
					eeprom_write_byte (&eebyte [18], 0x83);
					eeprom_write_byte (&eebyte [19], 0x90);
					eeprom_write_byte (&eebyte [20], 0x47);
					eeprom_write_byte (&eebyte [21], 0x81);
					eeprom_write_byte (&eebyte [22], 0xC3);
					eeprom_write_byte (&eebyte [23], 0x9A);
					eeprom_write_byte (&eebyte [24], 0x83);
					eeprom_write_byte (&eebyte [25], 0x94);
					eeprom_write_byte (&eebyte [26], 0x43);
					eeprom_write_byte (&eebyte [27], 0x82);
					eeprom_write_byte (&eebyte [28], 0xC3);
					eeprom_write_byte (&eebyte [29], 0x90);
					eeprom_write_byte (&eebyte [30], 0x84);
					eeprom_write_byte (&eebyte [31], 0x90);
					eeprom_write_byte (&eebyte [32], 0x11);
					eeprom_write_byte (&eebyte [33], 0x28);
					eeprom_write_byte (&eebyte [34], 0x18);
					eeprom_write_byte (&eebyte [35], 0x28);
					eeprom_write_byte (&eebyte [36], 0x34);
					eeprom_write_byte (&eebyte [37], 0x71);
					eeprom_write_byte (&eebyte [38], 0x54);
					eeprom_write_byte (&eebyte [39], 0x59);
					eeprom_write_byte (&eebyte [40], 0x26);
					eeprom_write_byte (&eebyte [41], 0x34);
					eeprom_write_byte (&eebyte [42], 0x69);
					eeprom_write_byte (&eebyte [43], 0x54);
					eeprom_write_byte (&eebyte [44], 0x59);
					eeprom_write_byte (&eebyte [45], 0x26);
					eeprom_write_byte (&eebyte [46], 0x34);
					eeprom_write_byte (&eebyte [47], 0xFF);
					eeprom_write_byte (&eebyte [48], 0x54);
					eeprom_write_byte (&eebyte [49], 0xCE);
					eeprom_write_byte (&eebyte [50], 0x71);
					eeprom_write_byte (&eebyte [51], 0x33);
					eeprom_write_byte (&eebyte [52], 0x22);
					eeprom_write_byte (&eebyte [53], 0xCC);
					eeprom_write_byte (&eebyte [54], 0x32);
					eeprom_write_byte (&eebyte [55], 0x40);
					eeprom_write_byte (&eebyte [56], 0x22);
					eeprom_write_byte (&eebyte [57], 0x71);
					eeprom_write_byte (&eebyte [58], 0x54);
					eeprom_write_byte (&eebyte [59], 0xCE);
					eeprom_write_byte (&eebyte [60], 0x34);
					eeprom_write_byte (&eebyte [61], 0x39);
					eeprom_write_byte (&eebyte [62], 0xFF);
					eeprom_write_byte (&eebyte [63], 0xFF);
					eeprom_write_byte (&eebyte [64], 0x86);
					eeprom_write_byte (&eebyte [65], 0xD0);
					eeprom_write_byte (&eebyte [66], 0x40);
					eeprom_write_byte (&eebyte [67], 0x71);
					eeprom_write_byte (&eebyte [68], 0x54);
					eeprom_write_byte (&eebyte [69], 0x23);
					eeprom_write_byte (&eebyte [70], 0xCD);
					eeprom_write_byte (&eebyte [71], 0x34);
					eeprom_write_byte (&eebyte [72], 0xD8);
					eeprom_write_byte (&eebyte [73], 0x40);
					eeprom_write_byte (&eebyte [74], 0x54);
					eeprom_write_byte (&eebyte [75], 0x3B);
					eeprom_write_byte (&eebyte [76], 0xFF);
					eeprom_write_byte (&eebyte [77], 0xFF);
					eeprom_write_byte (&eebyte [78], 0xFF);
					eeprom_write_byte (&eebyte [79], 0xFF);
					eeprom_write_byte (&eebyte [80], 0x4F);
					eeprom_write_byte (&eebyte [81], 0x93);
					eeprom_write_byte (&eebyte [82], 0x45);
					eeprom_write_byte (&eebyte [83], 0x53);
					eeprom_write_byte (&eebyte [84], 0x19);
					eeprom_write_byte (&eebyte [85], 0x11);
					eeprom_write_byte (&eebyte [86], 0x21);
					eeprom_write_byte (&eebyte [87], 0x19);
					eeprom_write_byte (&eebyte [88], 0x11);
					eeprom_write_byte (&eebyte [89], 0x21);
					eeprom_write_byte (&eebyte [90], 0x19);
					eeprom_write_byte (&eebyte [91], 0x11);
					eeprom_write_byte (&eebyte [92], 0x20);
					eeprom_write_byte (&eebyte [93], 0xB4);
					eeprom_write_byte (&eebyte [94], 0x10);
					eeprom_write_byte (&eebyte [95], 0xE0);
					eeprom_write_byte (&eebyte [96], 0x23);
					eeprom_write_byte (&eebyte [97], 0xCE);
					eeprom_write_byte (&eebyte [98], 0x32);
					eeprom_write_byte (&eebyte [99], 0x23);
					eeprom_write_byte (&eebyte [100], 0xCC);
					eeprom_write_byte (&eebyte [101], 0x31);
					eeprom_write_byte (&eebyte [102], 0xE0);
					eeprom_write_byte (&eebyte [103], 0xFF);
					eeprom_write_byte (&eebyte [104], 0x23);
					eeprom_write_byte (&eebyte [105], 0xCF);
					eeprom_write_byte (&eebyte [106], 0x32);
					eeprom_write_byte (&eebyte [107], 0x23);
					eeprom_write_byte (&eebyte [108], 0xCD);
					eeprom_write_byte (&eebyte [109], 0x31);
					eeprom_write_byte (&eebyte [110], 0xE0);
					eeprom_write_byte (&eebyte [111], 0xFF);
					eeprom_write_byte (&eebyte [112], 0xCC);
					eeprom_write_byte (&eebyte [113], 0x31);
					eeprom_write_byte (&eebyte [114], 0x40);
					eeprom_write_byte (&eebyte [115], 0x54);
					eeprom_write_byte (&eebyte [116], 0x23);
					eeprom_write_byte (&eebyte [117], 0xCE);
					eeprom_write_byte (&eebyte [118], 0x32);
					eeprom_write_byte (&eebyte [119], 0xCF);
					eeprom_write_byte (&eebyte [120], 0xE0);
					eeprom_write_byte (&eebyte [121], 0xCC);
					eeprom_write_byte (&eebyte [122], 0x33);
					eeprom_write_byte (&eebyte [123], 0x71);
					eeprom_write_byte (&eebyte [124], 0x23);
					eeprom_write_byte (&eebyte [125], 0xCC);
					eeprom_write_byte (&eebyte [126], 0x31);
					eeprom_write_byte (&eebyte [127], 0x3C);
					
					ff = 0;
				}
			}
		}
	}
	while (!0);
}



// Programmieren
void PROG()
{
	addr = 0;									// setze addr und prog auf 0
	prog = 0;
	do											// Hauptschleife
	{											// Adresse anzeigen
		adrlo = addr & 15;						// lade das untere Nibble der Adresse in adrlo
		PORTA = adrlo * 16 & 0xF0;				// gib das untere Nibble aus und warte 500 ms
		_delay_ms(300);
		PORTA = 0 & 0xF0;						// schalte die Anzeige fr 250 ms aus
		_delay_ms(200);
		byte = eeprom_read_byte (&eebyte [addr]);	// lese das eebyte-Array an der stelle addr aus
		dat = byte & 15;						// schreibe das unter Nibble in dat und
		kom = byte / 16;						// das obere Nibble in kom
		PORTA = kom * 16 & 0xF0;				// Befehl anzeigen - gib kom aus
		loop_until_bit_is_set (PINB, S2);		// warte bis die Taste S2 nicht mehr gedrckt wird
		_delay_ms(40);							// Taste entprellen
		prog = 1;								// setze prog = 1
		do										
		{
			if (!(PINB & (1<<S1)))				// wenn Taste S1 gedrckt wird, verzweige nach hier
			{
				if (prog == 1)					// Phase 1: Befehl anzeigen
				{								// wenn prog = 1 dann setzte prog = 2 und kom = 15
					prog = 2;
					kom = 15;				
				}
				if (prog == 2)					// Phase 2: Befehl verndern
				{								// wenn prog = 2 dann addier 1 zu kom und verknpfe
					kom = kom + 1;				// kom & 15, gib kom aus
					kom = kom & 15;
					PORTA = kom * 16 & 0xF0;
				}
				if (prog == 3)					// Phase 3: Befehl unverndert, Daten ndern
				{								// wenn prog = 3 dann setzte prog = 5 und dat = 15
					prog = 5;
					dat = 15;
				}
				if (prog == 4)					// Phase 4: Befehl und Daten gendert
				{								// wenn prog = 4 dann setzte prog = 5 und dat = 15
					prog = 5;
					dat = 15;
				}
				if (prog == 5)					// Phase 5: Daten verndern
				{								// wenn prog = 5 dann addier 1 zu dat und verknpfe
					dat = dat + 1;				// dat & 15, gib dat aus
					dat = dat & 15;
					PORTA = dat * 16 & 0xF0;
				}
				_delay_ms(40);					// Taste entprellen
				loop_until_bit_is_set (PINB, S1);	// Warte bis die Taste S1 nicht mehr gedrckt wird 
				_delay_ms(40);					// Taste entprellen
			}
			if (!(PINB & (1<<S2)))				// wenn Taste S2 gedrckt wird verzweige hier
			{
				if (prog == 3)					// nur angezeigt nicht verndert
				{								// wenn prog = 3 dann setzte prog = 7
					prog = 7;
				}
				if (prog == 1)					// wenn prog = 1 dann zeige dat an und setze prog = 3
				{
					PORTA = dat * 16 & 0xF0;
					prog = 3;
				}
				if (prog == 4)					// wenn prog = 4 dann zeige dat an u. setze prog = 6 
				{	
					PORTA = dat * 16 & 0xF0;
					prog = 6;
				}
				if (prog == 2)					// wenn prog = 2 dann zeige dat an u. setzte prog = 4
				{
					PORTA = dat * 16 & 0xF0;
					prog = 4;
				}
				if (prog == 6)					// nur Befehl wurde verndert
				{								// verknpfe dat & 15 und nehme kom * 16
					dat = dat & 15;				// schreibe beides in byte und schreibe das ins eeprom
					byte = kom * 16;			// an die stelle addr ins eebyte-Array
					byte = byte + dat;
					eeprom_write_byte (&eebyte [addr], byte);
					PORTA = 0 & 0xF0;			// schalte die Anzeige fr 600 ms aus
					_delay_ms(600);
					addr = addr + 1;			// addiere 1 zu addr und setzte prog = 0
					prog = 0;
				}
				if (prog == 5)					// Daten wurden verndert
				{								// verknpfe dat & 15 und nehme kom * 16
					dat = dat & 15;				// schreibe beides in byte und schreibe das ins eeprom
					byte = kom * 16;			// an die stelle addr ins eebyte-Array
					byte = byte + dat;
					eeprom_write_byte (&eebyte [addr], byte);
					PORTA = 0 & 0xF0;			// schalte die Anzeige fr 600 ms aus
					_delay_ms(600);
					addr = addr + 1;			// addiere 1 zu addr und setzte prog = 0
					prog = 0;					
				}
				if (prog == 7)					// wenn prog = 7 dann addiere 1 zu addr u. setze prog = 0
				{
					addr = addr + 1;
					prog = 0;
				}
				_delay_ms(40);					// Taste entprellen
				loop_until_bit_is_set (PINB, S2);	// Warte bis die Taste S2 nicht mehr gedrckt wird 
				_delay_ms(40);					// Taste entprellen
			}
		}
		while(!prog == 0);
	}
	while (!0);
}


		
// Testprogramm
void TEST()
{
	do
	{
		if (!(PINB & (1<<S1)))		// Wenn S1 geschaltet ist dann
		{							// schalte A1 bis A4 an.
			PORTA |= (1<<A1);
			_delay_ms(200);
			PORTA |= (1<<A2);
			_delay_ms(200);
			PORTA |= (1<<A3);
			_delay_ms(200);
			PORTA |= (1<<A4);
			_delay_ms(200);
		}

		if (!(PINB & (1<<S2)))		// Wenn S2 geschaltet ist dann
		{							// schalte A1 bis A4 aus.
			PORTA &= ~(1<<A1);
			_delay_ms(200);
			PORTA &= ~(1<<A2);
			_delay_ms(200);
			PORTA &= ~(1<<A3);
			_delay_ms(200);
			PORTA &= ~(1<<A4);
			_delay_ms(200);
		}

		if (!(PINA & (1<<E1)))		// Wenn E1 geschaltet ist dann
		{							// dann schalte PWM1 und PWM2
			_delay_ms(200);			// auf 50 und dann auf 100
			PWM1 = 50;
			_delay_ms(200);
			PWM2 = 50;
			_delay_ms(200);
			PWM1 = 100;
			_delay_ms(200);
			PWM2 = 100;
		}

		if (!(PINA & (1<<E2)))		// Wenn E2 geschaltet ist dann
		{							// dann schalte PWM1 und PWM2
			_delay_ms(200);			// auf 150, 200 und dann auf 250
			PWM1 = 150;
			_delay_ms(200);
			PWM2 = 150;
			_delay_ms(200);
			PWM1 = 200;
			_delay_ms(200);
			PWM2 = 200;
			_delay_ms(200);
			PWM1 = 250;
			_delay_ms(200);
			PWM2 = 250;
		}	
	
		if (!(PINA & (1<<E3)))		// Wenn E3 geschaltet ist dann
		{							// dann frage ADC1 ab und setzte PWM1
			PWM1 = ADC1();
		}

		if (!(PINA & (1<<E4)))		// Wenn E4 geschaltet ist dann
		{							// dann frage ADC2 ab und setzte PWM2
			PWM2 = ADC2();
		}
	}
	while (!0);
}





